@@ -54,4 +54,4 @@ from .sftp import SEEK_SET, SEEK_CUR, SEEK_END
 from .stream import SSHReader, SSHWriter
 
 # Import these explicitly to trigger register calls in them
-from . import curve25519, ed25519, ec, rsa, dsa, dh
+from . import ed25519, ecdsa, rsa, dsa, ecdh, dh
