@@ -69,6 +69,7 @@ class SSHChannel(SSHPacketHandler):
         self._recv_pktsize = max_pktsize
         self._recv_paused = True
         self._recv_buf = []
+        self._recv_partial = {}
 
         self._open_waiter = None
         self._request_waiters = []
@@ -182,8 +183,12 @@ class SSHChannel(SSHPacketHandler):
         """Deliver incoming data to the session"""
 
         if data == _EOF:
-            if not self._session.eof_received():
-                self.close()
+            if datatype in self._recv_partial:
+                raise DisconnectError(DISC_PROTOCOL_ERROR,
+                                      'Unicode decode error')
+
+            if not self._session.eof_received() and self._send_state == 'open':
+                self.write_eof()
         else:
             self._recv_window -= len(data)
 
@@ -194,13 +199,33 @@ class SSHChannel(SSHPacketHandler):
                 self._recv_window = self._init_recv_window
 
             if self._encoding:
-                try:
-                    data = data.decode(self._encoding)
-                except UnicodeDecodeError:
-                    raise DisconnectError(DISC_PROTOCOL_ERROR,
-                                          'Unicode decode error') from None
+                if datatype in self._recv_partial:
+                    encdata = self._recv_partial.pop(datatype) + data
+                else:
+                    encdata = data
 
-            self._session.data_received(data, datatype)
+                while encdata:
+                    try:
+                        data = encdata.decode(self._encoding)
+                        encdata = b''
+                    except UnicodeDecodeError as exc:
+                        if exc.start > 0:
+                            # Avoid pylint false positive
+                            # pylint: disable=invalid-slice-index
+                            data = encdata[:exc.start].decode()
+                            encdata = encdata[exc.start:]
+                        elif exc.reason == 'unexpected end of data':
+                            break
+                        else:
+                            raise DisconnectError(DISC_PROTOCOL_ERROR,
+                                                  'Unicode decode error')
+
+                    self._session.data_received(data, datatype)
+
+                if encdata:
+                    self._recv_partial[datatype] = encdata
+            else:
+                self._session.data_received(data, datatype)
 
     def _accept_data(self, data, datatype=None):
         """Accept new data on the channel
@@ -378,6 +403,7 @@ class SSHChannel(SSHPacketHandler):
         # If we haven't yet sent a close, send one now
         if self._send_state not in {'close_sent', 'closed'}:
             self._send_packet(MSG_CHANNEL_CLOSE)
+            self._send_state = 'close_sent'
 
         self._loop.call_soon(self._cleanup)
 
